@@ -2,6 +2,7 @@ class PyArdError(Exception):
     """
     Base Class for All py-ard Errors
     """
+
     pass
 
 
@@ -22,8 +23,9 @@ class InvalidMACError(PyArdError):
 
 
 class InvalidTypingError(PyArdError):
-    def __init__(self, message: str) -> None:
+    def __init__(self, message: str, cause=None) -> None:
         self.message = message
+        self.cause = cause
 
     def __str__(self) -> str:
         return f"Invalid HLA Typing: {self.message}"
